import { ElementRef, Injectable, SecurityContext } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';
import Prism from 'prismjs';
import * as _ from 'lodash-es';
export class PrismService {
    constructor(sanitizer) {
        this.sanitizer = sanitizer;
        this.templateOptions = { interpolate: /{{([\s\S]+?)}}/g };
    }
    highlight(el, options) {
        if (el instanceof ElementRef) {
            if (options.code) {
                el.nativeElement.innerHTML = this.sanitizer.sanitize(SecurityContext.HTML, this.escapeHtml(options.code));
            }
            if (options.interpolation) {
                el.nativeElement.innerHTML = this.interpolate(el.nativeElement.innerHTML, options.interpolation);
            }
            Prism.highlightElement(el.nativeElement, options.async, options.callback);
        }
    }
    hooks() {
        return Prism.hooks;
    }
    escapeHtml(unsafe) {
        return unsafe
            .replace(/&/g, '&amp;')
            .replace(/</g, '&lt;')
            .replace(/>/g, '&gt;')
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&#039;');
    }
    interpolate(string, interpolation) {
        if (interpolation && typeof interpolation === 'object') {
            return _.template(string, this.templateOptions)(interpolation);
        }
        return string;
    }
}
PrismService.decorators = [
    { type: Injectable },
];
PrismService.ctorParameters = () => [
    { type: DomSanitizer, },
];
//# sourceMappingURL=prism.service.js.map